package ePurse;

import javacard.framework.*;

public class EPursePin extends Applet {
	
	public final static byte EPURSE_CLA = (byte)0xC0;
	public final static byte EPURSE_ADD = (byte)0xB0;
	public final static byte EPURSE_SUB = (byte)0xB2;
	public final static byte EPURSE_BAL = (byte)0xB4;
	public final static byte EPURSE_PIN = (byte)0x20;	/* Check PIN */
	public final static byte EPURSE_NEW = (byte)0x21;	/* New PIN */
	public final static byte EPURSE_ULK = (byte)0x22;	/* Unlock */
	public final static byte MAX_TRIES = 3;
	public final static byte PIN_SIZE = 4;
	public final static byte[] DEFAULT_PIN = {0, 0, 0, 0};
	public final static byte[] UNLOCK_PIN = {1, 2, 3, 4};
	
	private short balance;
	private OwnerPIN thePIN;
	private OwnerPIN unlockPIN;
	
	public EPursePin(){
		balance = (short)0;
		thePIN = new OwnerPIN(MAX_TRIES, PIN_SIZE);
		thePIN.update(DEFAULT_PIN,(short)0,(byte)PIN_SIZE);
		unlockPIN = new OwnerPIN(MAX_TRIES, PIN_SIZE);
		unlockPIN.update(UNLOCK_PIN,(short)0,(byte)PIN_SIZE);
	}
	
	public static void install(byte []bAr, short bOff, byte bLen){
		EPursePin purse = new EPursePin();
		purse.register();
	}
	
	public boolean select(){
		if(unlockPIN.getTriesRemaining() == 0)
			return false;
		thePIN.reset();
		unlockPIN.reset();
		return true;
	}

	public void deselect(){
		thePIN.reset();
		unlockPIN.reset();
	}
	
	public void process(APDU apdu){
		byte []buffer = apdu.getBuffer();
		
		if(buffer[ISO7816.OFFSET_CLA] == ISO7816.CLA_ISO7816 &&
			buffer[ISO7816.OFFSET_INS] == ISO7816.INS_SELECT)
		return;
		
		if(buffer[ISO7816.OFFSET_CLA] != EPURSE_CLA)
			ISOException.throwIt(ISO7816.SW_CLA_NOT_SUPPORTED);
		
		if(thePIN.isValidated())
		{
			short amount = (short)0;
			switch(buffer[ISO7816.OFFSET_INS]){
				case EPURSE_ADD:
					apdu.setIncomingAndReceive();
					amount = Util.getShort(buffer,ISO7816.OFFSET_CDATA);
					balance += amount;
					break;
				case EPURSE_SUB:
					apdu.setIncomingAndReceive();
					amount = Util.getShort(buffer,ISO7816.OFFSET_CDATA);
					if(amount<=balance)
						balance -= amount;
					else
						ISOException.throwIt(ISO7816.SW_WRONG_DATA);
					break;
				case EPURSE_BAL:
					Util.setShort(buffer,ISO7816.OFFSET_CDATA,balance);
					apdu.setOutgoingAndSend(ISO7816.OFFSET_CDATA,(short)2);
					break;
				case EPURSE_NEW:
					apdu.setIncomingAndReceive();
					thePIN.update(buffer,ISO7816.OFFSET_CDATA,PIN_SIZE);
					break;
				default:
					ISOException.throwIt(ISO7816.SW_INS_NOT_SUPPORTED);
			}
		}
		else if(thePIN.getTriesRemaining() > 0)
		{
			switch(buffer[ISO7816.OFFSET_INS]){
				case EPURSE_PIN:
					apdu.setIncomingAndReceive();	
					if(!thePIN.check(buffer,ISO7816.OFFSET_CDATA,PIN_SIZE))
						ISOException.throwIt(ISO7816.SW_CONDITIONS_NOT_SATISFIED);
					break;
				case EPURSE_BAL:
					Util.setShort(buffer,ISO7816.OFFSET_CDATA,balance);
					apdu.setOutgoingAndSend(ISO7816.OFFSET_CDATA,(short)2);
					break;
				default:
					ISOException.throwIt(ISO7816.SW_INS_NOT_SUPPORTED);
			}
		}
		else if(unlockPIN.getTriesRemaining() > 0)
		{
			if(buffer[ISO7816.OFFSET_INS] == EPURSE_ULK)
			{
				apdu.setIncomingAndReceive();	
				if(!unlockPIN.check(buffer,ISO7816.OFFSET_CDATA,PIN_SIZE))
					ISOException.throwIt(ISO7816.SW_CONDITIONS_NOT_SATISFIED);
				else
				{
					thePIN.resetAndUnblock();
				}
			}
			else
				ISOException.throwIt(ISO7816.SW_INS_NOT_SUPPORTED);
		}
		else
			ISOException.throwIt(ISO7816.SW_INS_NOT_SUPPORTED);	
	}
}
		
